﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Drawing;
using System.Windows.Forms;

namespace CKMail
{
    ////////////////////////////////////////////////////////////////////////////////
    // [Common for CKMail]
    ////////////////////////////////////////////////////////////////////////////////
    //【変更履歴】 
    //
    // 2010-10-07 Ver1.1.0 監視インターバルを可変に変更。
    //            [F10]に[接続可能]ボタンを追加。
    //            [F10]に[FormEnabled]メソッドを追加。
    //
    // 2010-09-30 Ver1.0.0 効果音設定部分を追加。
    // 
    // 2010-09-29 Ver0.1.0 作成。
    // 
    ////////////////////////////////////////////////////////////////////////////////

    public class CC
    {
        //*** 定数
        public const int NUM_MIDFILE = 4;
        public const DialogResult RSLT_CANCEL = DialogResult.Cancel;
        public const DialogResult RSLT_OK = DialogResult.OK;
        public const string MSG_ERR = "ERR";
        public const string MSG_EXC = "EXC";
        public const string MSG_INF = "INF";

        public static string MidFile = String.Empty;
        public static string NL = Environment.NewLine;
        
        private const MessageBoxButtons BTN_OK = MessageBoxButtons.OK;
        private const MessageBoxButtons BTN_OKCANCEL = MessageBoxButtons.OKCancel;
        private const MessageBoxIcon ICON_ERR = MessageBoxIcon.Error;
        private const MessageBoxIcon ICON_EXC = MessageBoxIcon.Exclamation;
        private const MessageBoxIcon ICON_INF = MessageBoxIcon.Information;
        private const MessageBoxIcon ICON_QTN = MessageBoxIcon.Question;
        private const string MSG_TTL = "メール受信通知";        

        //*** グローバルメソッド

        /// <summary> [IsDigit]
        /// 引数文字列の全文字が数字である場合にtrue、否の場合falseを返す
        /// </summary>
        /// <param name="vData"></param>
        /// <returns>true:OK, false:引数文字列に数字以外のものが含まれている</returns>
        public static bool IsDigit(string vnum)
        {
            if (System.Text.RegularExpressions.Regex.IsMatch(vnum, "^[0-9]+$"))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary> [MsgCanceled]
        /// OKCancelのMessageBoxを表示し、結果がCancelの場合にtrueを返す
        /// </summary>
        /// <param name="vMsg"></param>
        /// <returns>true:DialogResult.Cancel, false:DialogResult.OK</returns>
        public static bool MsgCanceled(string vMsg)
        {
            if (MessageBox.Show(vMsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN) == RSLT_CANCEL)
                return true;
            else
                return false;
        }

        /// <summary> [MsgCanceled]：([MsgCanceled]のオーバーロード)
        /// </summary>
        /// <param name="vmsg"></param>
        /// <param name="vDefCancel">trueの場合、Button2をDefaultButtonにする</param>
        /// <returns>true:DialogResult.Cancel, false:DialogResult.OK</returns>
        public static bool MsgCanceled(string vmsg, bool vDefCancel)
        {
            if (vDefCancel)
            {
                if (MessageBox.Show(vmsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN,
                        MessageBoxDefaultButton.Button2) == RSLT_CANCEL)
                    return true;
                else
                    return false;
            }
            else
            {
                if (MessageBox.Show(vmsg, MSG_TTL, BTN_OKCANCEL, ICON_QTN,
                        MessageBoxDefaultButton.Button1) == RSLT_CANCEL)
                    return true;
                else
                    return false;
            }
        }

        /// <summary> [GetAllControls]
        /// コンテナコントロール内の全コントロールを配列で取得する
        /// 　.NET_Tips(2004/10/29)から取得
        /// </summary>
        /// <param name="top">コンテナコントロール</param>
        /// <returns>コントロールの配列</returns>
        public static Control[] GetAllControls(Control vtop)
        {
            List<Control> buf = new List<Control>();
            foreach (Control c in vtop.Controls)
            {
                buf.Add(c);
                buf.AddRange(GetAllControls(c));
            }
            return buf.ToArray();
        }

        /// <summary> [NByteCount]
        ///  引数文字列のバイト数を返す
        /// </summary>
        /// <param name="vstr"></param>
        /// <returns></returns>
        public static int NByteCount(string vstr)
        {
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
            return sjisEnc.GetByteCount(vstr);
        }

        /// <summary> [CheckDir]：ディレクトリの末尾が"\"でないときは、"\"を付ける
        /// </summary>
        /// <param name="vdir"></param>
        /// <returns></returns>
        public static string CheckDir(string vdir)
        {
            string result = vdir;
            if (!vdir.EndsWith("\\"))
                result += "\\";

            return result;
        }

        /// <summary> [NLeftB]：引数文字列の左から引数バイト数目で切ったものを返す
        /// 
        /// </summary>
        /// <param name="vstr"></param>
        /// <param name="vnum">左から切り出すバイト数</param>
        /// <param name="vinclude">
        /// 　バイト数が全角文字の中間にきたとき
        /// 　　true:その文字を含める、false:その文字を含めない
        /// </param>
        /// <returns></returns>
        public static string NLeftB(string vstr, int vnum, bool vinclude)
        {
            if (vnum <= 0)
                return String.Empty;

            if (vnum > NByteCount(vstr))
                return vstr;
            else
            {
                int sumByte = 0;
                for (int i = 0; i < vstr.Length; i++)
                {
                    string char1 = vstr.Substring(i, 1);
                    sumByte += NByteCount(char1);
                    if (sumByte == vnum)
                        return vstr.Substring(0, i + 1);
                    else if (sumByte > vnum)
                    {
                        if (i == 0 && !vinclude)
                            return String.Empty;
                        else if (vinclude)
                            return vstr.Substring(0, i + 1);
                        else
                            return vstr.Substring(0, i);
                    }
                }
                return vstr;
            }
        }

        /// <summary> [CtrlKeyDown]：[Enter]キーにより、コントロールのFocusを前後させる
        /// </summary>
        /// <param name="vForm"></param>
        /// <param name="e"></param>
        public static void CtrlKeyDown(Form vForm, KeyEventArgs e)
        {
            bool bMove;
            if (e.KeyCode == Keys.Enter && (!e.Control))
            {
                if (e.Modifiers == Keys.Shift)
                    bMove = false;
                else
                    bMove = true;

                e.SuppressKeyPress = true;
                vForm.SelectNextControl(vForm.ActiveControl, bMove, true, true, true);
            }
        }

        /// <summary> [SetForm]：FormのLocationをセンターより少し上にセットする
        /// </summary>
        /// <param name="form"></param>
        public static void SetForm(Form form)
        {
            form.Left = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Width - form.Width) / 2);
            form.Top = Convert.ToInt32((Screen.PrimaryScreen.Bounds.Height - form.Height) / 3);
        }

        /// <summary> [SetGridProperty]：DataGridViewのプロパティをセットする
        /// </summary>
        /// <param name="vgrid"></param>
        public static void SetGridProperty(DataGridView vgrid)
        {
            vgrid.AllowUserToAddRows = false;
            vgrid.AllowUserToDeleteRows = false;
            vgrid.AllowUserToOrderColumns = false;
            vgrid.AllowUserToResizeColumns = false;
            vgrid.AllowUserToResizeRows = false;
            vgrid.ColumnHeadersHeight = 25;
            vgrid.ColumnHeadersHeightSizeMode =
                DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            vgrid.ColumnHeadersDefaultCellStyle.Alignment =
                DataGridViewContentAlignment.MiddleCenter;
            vgrid.ColumnHeadersDefaultCellStyle.Font =
                new Font("ＭＳ ゴシック", 12, FontStyle.Bold);
            vgrid.DefaultCellStyle.Font = new Font("ＭＳ ゴシック", 12, FontStyle.Bold);
            vgrid.MultiSelect = false;
            vgrid.ReadOnly = true;
            vgrid.RowHeadersVisible = false;
            vgrid.SelectionMode = DataGridViewSelectionMode.FullRowSelect;
            //*** ColumnHeaderのClickによるSortを無効にする
            for (int i = 0; i < vgrid.Columns.Count; i++)
            {
                vgrid.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            }
        }

        /// <summary> [ShowMsg]：INFORMATION、EXCLAMATIONのMessageBoxを出す
        /// </summary>
        /// <param name="vmsg"></param>
        /// <param name="vmode"></param>
        public static void ShowMsg(string vmsg, string vmode)
        {
            if (vmode == MSG_INF)
                MessageBox.Show(vmsg, MSG_TTL, BTN_OK, ICON_INF);
            else if (vmode == MSG_ERR)
                MessageBox.Show(vmsg, MSG_TTL, BTN_OK, ICON_ERR);
            else
                MessageBox.Show(vmsg, MSG_TTL, BTN_OK, ICON_EXC);
        }
    }

}
