﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Collections.Specialized;
using System.Text.RegularExpressions;

namespace CKMail
{
    class CMailObject
    {
        private NameValueCollection _header = new NameValueCollection();

        /// <summary> [Header]プロパティ：メールヘッダのコレクション
        /// </summary>
        public NameValueCollection Header
        {
            get { return _header; }
        }

        /// <summary> [ParseHeader]メソッド：ヘッダ部をパースする
        /// </summary>
        public void ParseHeader()
        {
            string[] keys = _header.AllKeys;
            for (int i = 0; i < keys.Length; i++)
            {
                if (i == 9)
                {
                    int ij = i;
                }
                string[] values = _header.GetValues(keys[i]);
                // 削除
                _header.Remove(keys[i]);

                if (i == 9)
                {
                    int ij = i;
                }
                // デコードしたものを設定し直す
                for (int j = 0; j < values.Length; j++)
                {
                    // デコード処理
                    values[j] = HeaderDecode(values[j]).Trim();
                    if (keys[i].ToLower() == "date")
                    {
                        _header.Add(keys[i], DateAdjusted(values[j]));
                        break;
                    }
                    else if (keys[i].ToLower() == "from")
                    {
                        _header.Add(keys[i], CC.NLeftB(values[j], 30, false));
                        break;
                    }
                    else if (keys[i].ToLower() == "subject")
                    {
                        _header.Add(keys[i], CC.NLeftB(values[j], 80, false));
                        break;
                    }
                    else
                    {
                        _header.Add(keys[i], values[j]);
                    }
                }
            }       
        }


        /// <summary> [Base64Decode]メソッド：Base64デコードする
        ///  [HeaderReplacer]、[ParseBody]から呼出し
        /// </summary>
        /// <param name="vsrc"></param>
        /// <returns></returns>
        static byte[] Base64Decode(string vsrc)
        {
            try
            {
                return Convert.FromBase64String(vsrc);
            }
            catch
            {
                return Convert.FromBase64String(String.Empty);
            }
        }

        /// <summary> [QueueDecode]メソッド：Quote-Printableデコードする
        ///  [HeaderReplacer]、[ParseBody]から呼出し
        /// </summary>
        /// <param name="vsrc">デコードしたい文字列</param>
        /// <returns>デコードした結果のバイト列</returns>
        static byte[] QueueDecode(string vsrc)
        {
            byte[] b = new byte[vsrc.Length];
            int cnt = 0;
            for (int i = 0; i < vsrc.Length; i++)
            {
                switch (vsrc.Substring(i, 1))
                {
                    case "=":
                        try
                        {
                            // 次の2文字を16進数で読む
                            string test = vsrc.Substring(i + 1, 2);
                            b[cnt] = Convert.ToByte(vsrc.Substring(i + 1, 2), 16);
                            i += 2;
                        }
                        catch
                        {
                            //
                        }
                        break;

                    case "_":
                        // 空白にする
                        b[cnt] = 0x20;
                        break;
                    default:
                        // そのままの値を採用
                        Encoding.ASCII.GetBytes(vsrc, i, 1, b, cnt);
                        break;
                }
                cnt++;
            }
            return b;
        }

        /// <summary> [HeaderReplacer]メソッド
        ///  HeaderDecodeから呼び出される正規表現でマッチしたときのコールバック
        ///  [HeaderDecode]から呼出し
        /// </summary>
        /// <param name="vmat"></param>
        /// <returns></returns>
        static string HeaderReplacer(Match vmat)
        {
            // 置き換える
            string enctype = vmat.Groups[1].ToString();
            string encstr = vmat.Groups[3].ToString();
            if (vmat.Groups[2].ToString().ToLower() == "q")
            {
                // Qデコード
                return Encoding.GetEncoding(enctype).GetString(QueueDecode(encstr));
            }
            else
            {
                // Base64デコード
                return Encoding.GetEncoding(enctype).GetString(Base64Decode(encstr));
            }
        }

        /// <summary> [DateAdjusted]メソッド
        /// [ParseHeader]から呼出し
        /// </summary>
        /// <param name="vdate"></param>
        /// <returns></returns>
        private string DateAdjusted(string vdate)
        {
            const int I_DIF = 9;    // UTCとJSTの時差

            string result = String.Empty;
            try
            {
                string[] splitter = { " " };
                string[] adate = vdate.Split(splitter,
                        StringSplitOptions.RemoveEmptyEntries);
                string sdate = adate[0];
                for (int i = 1; i < 5; i++)
                {
                    sdate += " " + adate[i];
                }

                DateTime target = Convert.ToDateTime(sdate);
                try
                {
                    if (5 < adate.Length)
                    {
                        // 時差の補正
                        int itdiff = I_DIF - (int)(Int32.Parse(adate[5]) / 100);
                        target = target.AddHours(itdiff);
                    }

                    //int i7 = 16;
                }
                catch
                {
                    //
                }
                result = target.ToString("yyyy/MM/dd HH:mm:ss");
            }
            catch
            {
                //
            }
            return result;            
        }

        /// <summary> [HeaderDecode]メソッド：ヘッダ部をデコードする
        ///  [ParseHeader]から呼出し
        /// </summary>
        /// <param name="vsrc">デコードすべきヘッダの文字列</param>
        /// <returns>デコード後の文字列</returns>
        private string HeaderDecode(string vsrc)
        {
            // QデコードまたはBase64デコードする
            return Regex.Replace(vsrc, "=\\?([^\\?]+)\\?([qQbB])\\?([^\\?]+)\\?=",
                    new MatchEvaluator(HeaderReplacer));
        }
        
    }
}
