﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace CKMail
{
    public partial class F00_Main : Form
    {
        /// <summary> コンストラクター
        /// </summary>
        public F00_Main()
        {
            InitializeComponent();
        }

        /// <summary> [FromWatch]プロパティ
        /// </summary>
        public bool FromWatch
        {
            set { _fromWatch = value; }
        }

        /// <summary> [ListMsgID]プロパティ
        /// </summary>
        public List<string> ListMsgID
        {
            set { _lstMsgIDOld = value; }
        }

        const string COL_DTE = "受信日時";
        const string COL_FRM = "差出人";
        const string COL_MID = "MessageID";
        const string COL_NUM = "No";
        const string COL_SJT = "件名";

        private bool _fromWatch = false;
        private bool _canWatch = false;       // 監視状態に入れるかを示すフラグ
        private DataTable _dt;
        private List<CMailObject> _lstMailObj;                // [GetMailObjList]で取得
        private List<string> _lstMsgID = new List<string>();
        private List<string> _lstMsgIDOld;
        private string _popserver;
        private string _account;
        private string _password;
        private string _popport;

        /// <summary> [Form_Closed]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closed(object sender, FormClosedEventArgs e)
        {
            try
            {
                Application.Exit();
            }
            catch
            {
                //
            }
        }

        /// <summary> [Form_Load]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            this.Size = Properties.Settings.Default.F00Size;
            CC.SetForm(this);

            string ver =
                System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString();
            this.Text += " (Ver " + ver + ")";

            InitializeForm();
            if (_fromWatch)
            {
                // [F20_WatchMail]から戻った時の処理
                MenuViewClick();
                SelectNewRow();
                _fromWatch = false;
            }
        }

        /// <summary> [Form_Resize]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Resize(object sender, EventArgs e)
        {
            Properties.Settings.Default.F00Size = this.Size;
            Properties.Settings.Default.Save();
        }

        /// <summary> [終了]メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuEnd_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary> [基本設定]メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuSetBase_Click(object sender, EventArgs e)
        {
            var envform = new F10_SetBase();
            envform.ShowDialog();
            if (envform.DialogResult == CC.RSLT_OK)
            {
                SetServerProperty();
                _dt.Rows.Clear();
                grdView.DataSource = null;
                _canWatch = false;
            }
            envform.Dispose();
        }

        /// <summary> [効果音設定]メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuSetSound_Click(object sender, EventArgs e)
        {
            var soundform = new F12_SetSound();
            soundform.ShowDialog();
            soundform.Dispose();
        }

        /// <summary> [監視]メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuWatch_Click(object sender, EventArgs e)
        {
            if (!IsSettingsOK()) return;
            if (!_canWatch)
            {
                CC.ShowMsg("先に[一覧表示]メニューをクリックしてください。",
                    CC.MSG_EXC);
                return;
            }

            if (!CC.MsgCanceled("監視画面をタスクバーに最小化して監視を行います。"))
            {
                var nform = new F20_WatchMail();
                nform.ListMsgID = _lstMsgID;
                nform.Show();
                this.Hide();
            }
        }

        /// <summary> [一覧表示]メニュークリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void mnuView_Click(object sender, EventArgs e)
        {
            MenuViewClick();
        }


        /// <summary> [IsSettingsOK]メソッド
        /// [mnuWatch_Click]、[MenuViewClick]から呼出し
        /// </summary>
        /// <returns></returns>
        private bool IsSettingsOK()
        {
            if (_popserver.Length == 0)
            {
                CC.ShowMsg("POP3サーバー名が未入力です。", CC.MSG_EXC);
                return false;
            }
            else if (_account.Length == 0)
            {
                CC.ShowMsg("アカウント名が未入力です。", CC.MSG_EXC);
                return false;
            }
            else if (_password.Length == 0)
            {
                CC.ShowMsg("パスワードが未入力です。", CC.MSG_EXC);
                return false;
            }
            else if (!CC.IsDigit(_popport))
            {
                CC.ShowMsg("ポート番号が不適切です。", CC.MSG_EXC);
                return false;
            }
            return true;
        }

        /// <summary> [Removeaddress]メソッド：差出人からアドレス部分を除く
        /// [ShowMailList]から呼出し
        /// </summary>
        /// <param name="vfrom"></param>
        /// <returns></returns>
        private string RemoveAddress(string vfrom)
        {
            int position = vfrom.IndexOf("<");
            string from = String.Empty;
            if (position < 1)
            {
                from = vfrom;
            }
            else
            {
                string from0 = vfrom.Substring(0, position).Trim();
                if (from0.Length == 0)
                    from = vfrom;
                else
                    from = from0;
            }
            return from;
        }

        /// <summary> [GetMailObjList]メソッド
        /// [ShowMailList]から呼出し
        /// </summary>
        private void GetMailObjList(ref bool visConnect)
        {
            _lstMailObj = new List<CMailObject>();
            var pop = new CPop();
            int port = Int32.Parse(_popport);
            if (pop.IsConnectOK(_popserver, port, _account, _password, false))
            {
                // 一覧取得
                _lstMailObj = pop.MailObjList();

                // 切断
                pop.Quit();
                visConnect = true;
            }
            else
            {
                visConnect = false;
            }
            pop = null;
        }

        /// <summary> [InitializeForm]メソッド
        /// </summary>
        private void InitializeForm()
        {
            SetMidFile();
            SetServerProperty();

            // _dtの初期化
            _dt = new DataTable();
            _dt.Columns.Add(COL_NUM);
            _dt.Columns.Add(COL_FRM);
            _dt.Columns.Add(COL_SJT);
            _dt.Columns.Add(COL_DTE);
            _dt.Columns.Add(COL_MID);
        }

        /// <summary> [MenuViewClick]イベント
        /// [Form_Load]、[mnuSetBase_Click]、[mnuView_Click]から呼出し
        /// </summary>
        private void MenuViewClick()
        {
            if (!IsSettingsOK()) return;

            this.Cursor = Cursors.WaitCursor;
            ShowMailList();

            this.Cursor = Cursors.Default;
        }

        /// <summary> [SelectNewRow]メソッド
        /// [Form_Load]から呼出し([F20_WatchMail]から戻った時の処理)
        /// 　⇒ 当初リストにない行を選択行にする
        /// </summary>
        private void SelectNewRow()
        {
            int countNew = grdView.Rows.Count;
            int countOld = _lstMsgIDOld.Count;
            if (countNew == 0) return;

            for (int i = 0; i < countNew; i++)
            {
                if (countOld == 0)
                {
                    grdView.Rows[i].Selected = true;
                }
                else
                {
                    bool result = true;
                    for (int j = 0; j < countOld; j++)
                    {
                        if (grdView.Rows[i].Cells[COL_MID].Value.ToString()
                            == _lstMsgIDOld[j])
                        {
                            result = false;
                            break;
                        }
                    }
                    grdView.Rows[i].Selected = result;
                }
            }
        }

        /// <summary> [SetMidFile]メソッド
        /// [InitializeForm]から呼出し
        /// </summary>
        private void SetMidFile()
        {
            switch (Properties.Settings.Default.MidFileIdx)
            {
                case 0:
                    CC.MidFile = Properties.Settings.Default.MidFile0;
                    break;
                case 1:
                    CC.MidFile = Properties.Settings.Default.MidFile1;
                    break;
                case 2:
                    CC.MidFile = Properties.Settings.Default.MidFile2;
                    break;
                case 3:
                    CC.MidFile = Properties.Settings.Default.MidFile3;
                    break;
            }
        }

        /// <summary> [SetServerProperty]メソッド
        /// [mnuSetting_Click]、[InitializeForm]から呼出し
        /// </summary>
        private void SetServerProperty()
        {
            _popserver = Properties.Settings.Default.PopServer;
            _account = Properties.Settings.Default.Account;
            _password = Properties.Settings.Default.Password;
            _popport = Properties.Settings.Default.PopPort;
        }

        /// <summary> [ShowMailList]メソッド：MailListをグリッドに表示する
        /// [MenuViewClick]から呼出し
        /// </summary>
        /// <param name="vmsg"></param>
        private void ShowMailList()
        {
            const string HED_DATE = "date";
            const string HED_FROM = "from";
            const string HED_MSID = "Message-ID";
            const string HED_SBJT = "subject";

            if (grdView.DataSource != null) grdView.DataSource = null;

            _dt.Rows.Clear();
            bool isConnectionOK = true;
            GetMailObjList(ref isConnectionOK);
            if (!isConnectionOK)
            {
                _canWatch = false;
                return;
            }

            _canWatch = true;
            if (_lstMailObj.Count == 0)
            {
                if (!_fromWatch)
                    CC.ShowMsg("現在、リストにメールはありません。", CC.MSG_INF);
                return;
            }

            int count = 0;
            foreach (CMailObject mobject in _lstMailObj)
            {
                _dt.Rows.Add();
                _dt.Rows[count][COL_FRM] = RemoveAddress(mobject.Header[HED_FROM]);
                _dt.Rows[count][COL_SJT] = mobject.Header[HED_SBJT];
                _dt.Rows[count][COL_DTE] = mobject.Header[HED_DATE];
                _dt.Rows[count][COL_MID] = mobject.Header[HED_MSID];
                _lstMsgID.Add(mobject.Header[HED_MSID]);
                count += 1;
            }

            if (_dt.Rows.Count > 0)
            {
                grdView.DataSource = _dt;
                CC.SetGridProperty(grdView);
                grdView.MultiSelect = true;
                grdView.Columns[COL_MID].Visible = false;
                grdView.AutoResizeColumns();
                grdView.Columns[COL_NUM].DefaultCellStyle.Alignment
                    = DataGridViewContentAlignment.MiddleRight;
                grdView.Columns[COL_FRM].Width = 250;
                grdView.Sort(grdView.Columns[COL_DTE],
                    System.ComponentModel.ListSortDirection.Descending);
                // grdViewのCOL_NUM列にNoを追加する
                for (int i = 0; i < grdView.Rows.Count; i++)
                {
                    grdView.Rows[i].Cells[COL_NUM].Value = i + 1;
                }
                grdView.ClearSelection();
            }
            else
            {
                CC.ShowMsg("現在、リストにメールはありません。", CC.MSG_INF);
            }
        }

    }
}
