﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace CKMail
{
    public partial class F10_SetBase : Form
    {
        /// <summary> コンストラクター
        /// </summary>
        public F10_SetBase()
        {
            InitializeComponent();
        }

        /// <summary> [接続確認]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCheck_Click(object sender, EventArgs e)
        {
            if (IsConnectionOK()) CC.ShowMsg("接続可能です。", CC.MSG_INF);
        }

        /// <summary> [OK]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            // 入力値の形式確認
            if (!IsConnectionOK()) return;

            // 設定実行の確認
            if (CC.MsgCanceled("この設定を保存します。")) return;                
            
            // 入力値の保存
            Properties.Settings.Default.PopServer = txtPopServer.Text.Trim();
            Properties.Settings.Default.Account = txtAccount.Text.Trim();
            Properties.Settings.Default.Password = txtPassword.Text.Trim();
            Properties.Settings.Default.PopPort = txtPopPort.Text.Trim();
            Properties.Settings.Default.Interval = (int)nudInterval.Value;
            Properties.Settings.Default.Save();

            // 呼出しFormへ戻る
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        /// <summary> [Form_Load]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            CC.SetForm(this);
            InitializeForm();
        }

        /// <summary> [TB_Enter]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TB_Enter(object sender, EventArgs e)
        {
            ((TextBox)sender).SelectAll();
        }

        /// <summary> [TB_KeyDown]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void TB_KeyDown(object sender, KeyEventArgs e)
        {
            CC.CtrlKeyDown(this, e);
        }


        /// <summary> [IsConnectionOK]メソッド：接続設定の確認
        /// [btnCheck_Click]、[btnOK_Click]から呼出し
        /// </summary>
        /// <returns></returns>
        private bool IsConnectionOK()
        {
            string server = txtPopServer.Text.Trim();
            string account = txtAccount.Text.Trim();
            string password = txtPassword.Text.Trim();
            string port = txtPopPort.Text.Trim();

            if (server.Length == 0)
            {
                CC.ShowMsg("POP3サーバー名が未入力です。", CC.MSG_EXC);
                txtPopServer.Focus();
                return false;
            }
            else if (account.Length == 0)
            {
                CC.ShowMsg("アカウント名が未入力です。", CC.MSG_EXC);
                txtAccount.Focus();
                return false;
            }
            else if (password.Length == 0)
            {
                CC.ShowMsg("パスワードが未入力です。", CC.MSG_EXC);
                txtPassword.Focus();
                return false;
            }
            else if (!CC.IsDigit(port))
            {
                CC.ShowMsg("ポート番号が不適切です。", CC.MSG_EXC);
                txtPopPort.Focus();
                return false;
            }

            // 入力値によるサーバー接続の確認
            FormEnabled(false);
            var pop = new CPop();
            int iport = Int32.Parse(port);
            if (!pop.IsConnectOK(server, iport, account, password, false))
            {
                pop = null;
                FormEnabled(true);
                return false;
            }
            pop.Quit();
            pop = null;
            FormEnabled(true);
            return true;
        }

        /// <summary> [FormEnabled]メソッド
        /// </summary>
        /// <param name="venable"></param>
        private void FormEnabled(bool venable)
        {
            this.Cursor = (venable) ? Cursors.Default : Cursors.WaitCursor;
            this.Enabled = venable;
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            txtPopServer.Text = Properties.Settings.Default.PopServer;
            txtAccount.Text = Properties.Settings.Default.Account;
            txtPassword.Text = Properties.Settings.Default.Password;
            txtPopPort.Text = Properties.Settings.Default.PopPort;
            nudInterval.Value = Properties.Settings.Default.Interval;
        }

    }
}
