﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.Win32;       // SystemEventsに使用

namespace CKMail
{
    public partial class F20_WatchMail : Form
    {
        /// <summary> コンストラクター
        /// </summary>
        public F20_WatchMail()
        {
            InitializeComponent();
        }

        /// <summary> [ListMsgID]プロパティ
        /// </summary>
        public List<string> ListMsgID
        {
            set { _lstMsgIDOld = value; }
        }

        const string LBL_DATE = "日　時：";
        const string LBL_FROM = "差出人：";
        const string LBL_SBJT = "件　名：";

        private bool _isNoSound;             // Setting情報 [InitializeForm]で取得
        private bool _existMidFile;
        private CMidi _cmidi;                // CMidiオブジェクト [CheckNewMail]で初期化
        private List<string> _lstMsgID;
        private List<string> _lstMsgIDOld;
                
        /// <summary> [一覧画面へ]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnBack_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary> [最小化]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMinimize_Click(object sender, EventArgs e)
        {
            ChangeText(true);
            this.WindowState = FormWindowState.Minimized;
            if (!timer1.Enabled) timer1.Enabled = true;
            if (_cmidi != null) _cmidi.StopMidi();
        }

        /// <summary> [Form_Closing]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closing(object sender, FormClosingEventArgs e)
        {
            if (_cmidi != null) _cmidi.StopMidi();
            SystemEvents.SessionEnding -=
                    new SessionEndingEventHandler(SystemEvents_SessionEnding);

            var mform = new F00_Main();
            mform.FromWatch = true;
            mform.ListMsgID = _lstMsgIDOld;
            mform.Show();
        }

        /// <summary> [Form_Load]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            CC.SetForm(this);

            InitializeForm();
            timer1.Interval = (Properties.Settings.Default.Interval) * 60000;
            timer1.Enabled = true;
            this.WindowState = FormWindowState.Minimized;

            SystemEvents.SessionEnding +=
                    new SessionEndingEventHandler(SystemEvents_SessionEnding);
        }

        /// <summary> [timer1_Tick]メソッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            CheckNewMail();
        }

        /// <summary> [SystemEvents_SessionEnding]イベント
        /// [Form_Closing]、[Form_Load]から呼出し
        /// 強制シャットダウンされたときのエラー発生を防止する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SystemEvents_SessionEnding(object sender, SessionEndingEventArgs e)
        {
            // シャットダウンされたときには、アプリケーションを止める
            while (Application.OpenForms.Count > 0)
            {
                Application.OpenForms[0].Close();
            }
            Application.Exit();
        }


        /// <summary> [IsNewID]メソッド
        /// [CheckNewMail]から呼出し
        /// </summary>
        /// <param name="vID"></param>
        /// <returns></returns>
        private bool IsNewID(string vID)
        {
            int count = _lstMsgID.Count;
            if (count > 0)
            {
                for (int i = 0; i < count; i++)
                {
                    if (vID.ToLower() == _lstMsgID[i].ToLower())
                        return false;
                }
            }
            return true;
        }

        /// <summary> [MailObjectList]メソッド
        /// [CheckNewMail]から呼出し
        /// </summary>
        /// <returns></returns>
        private List<CMailObject> MailObjList()
        {
            string popserver = Properties.Settings.Default.PopServer;
            string account = Properties.Settings.Default.Account;
            string password = Properties.Settings.Default.Password;
            int popport = Int32.Parse(Properties.Settings.Default.PopPort);

            var lstMailObj = new List<CMailObject>();
            var pop = new CPop();
            if (pop.IsConnectOK(popserver, popport, account, password, true))
            {
                // 一覧取得
                lstMailObj = pop.MailObjList();

                // 切断
                pop.Quit();
            }
            pop = null;

            return lstMailObj;
        }

        /// <summary> [ChangeText]メソッド
        /// [btnMinimize_Click]、[InitializeForm]から呼出し
        /// </summary>
        /// <param name="isWatch"></param>
        private void ChangeText(bool isWatch)
        {
            const string BUTN_GET = "ＯＫ";
            const string BUTN_WAT = "画面最小化";
            const string FORM_GET = "新着メール受信";
            const string FORM_WAT = "メール監視中";
            const string MAIN_GET = "新しいメールが届きました！！";
            const string MAIN_WAT = "メール受信を監視しています...";

            if (isWatch)
            {
                this.Text = FORM_WAT;
                lblMainMsg.Text = MAIN_WAT;
                lblMainMsg.Font = new Font(lblMainMsg.Font.Name, 12, FontStyle.Bold);
                btnMinimize.Text = BUTN_WAT;
                lblFrom.Text = LBL_FROM;
                lblSubject.Text = LBL_SBJT;
                lblDate.Text = LBL_DATE;
            }
            else
            {
                this.Text = FORM_GET;
                lblMainMsg.Text = MAIN_GET;
                lblMainMsg.Font = new Font(lblMainMsg.Font.Name, 18);
                btnMinimize.Text = BUTN_GET;
            }
            Application.DoEvents();
        }

        /// <summary> [CheckNewMail]メソッド
        /// [timer1_Tick]から呼出し
        /// </summary>
        private void CheckNewMail()
        {
            const string HED_DATE = "date";
            const string HED_FROM = "from";
            const string HED_MSID = "Message-ID";
            const string HED_SBJT = "subject";

            // MailObjectのリストを取得
            List<CMailObject> lstMailObj = MailObjList();
            if (lstMailObj.Count == 0) return;

            // 新規メールの確認
            for (int i = 0; i < lstMailObj.Count; i++)
            {
                if (IsNewID(lstMailObj[i].Header[HED_MSID]))
                {
                    timer1.Enabled = false;
                    ChangeText(false);
                    _lstMsgID.Add(lstMailObj[i].Header[HED_MSID]);
                    lblFrom.Text = LBL_FROM + lstMailObj[i].Header[HED_FROM];
                    lblSubject.Text = LBL_SBJT + lstMailObj[i].Header[HED_SBJT];
                    lblDate.Text = LBL_DATE + lstMailObj[i].Header[HED_DATE];
                    Application.DoEvents();
                    this.WindowState = FormWindowState.Normal;

                    // 効果音を発生させる
                    if (!_isNoSound && _existMidFile)
                    {
                        _cmidi = new CMidi(CC.MidFile);
                        _cmidi.CanStartMidi();
                    }

                    // 1件でも見つかったら、そこでbreakする
                    break;
                }
            }
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            ChangeText(true);

            // _lstMsgIDの初期化
            _lstMsgID = new List<string>();
            if (_lstMsgIDOld.Count > 0)
            {
                for (int i = 0; i < _lstMsgIDOld.Count; i++)
                {
                    _lstMsgID.Add(_lstMsgIDOld[i]);
                }
            }

            // Setting情報の取得
            _isNoSound = Properties.Settings.Default.NoSound;
            _existMidFile = System.IO.File.Exists(CC.MidFile);                       
        }

    }
}
